@@ -1,7 +1,7 @@
 /* Test file for mpfr_rint, mpfr_trunc, mpfr_floor, mpfr_ceil, mpfr_round,
    mpfr_rint_trunc, mpfr_rint_floor, mpfr_rint_ceil, mpfr_rint_round.
 
-Copyright 2002-2016 Free Software Foundation, Inc.
+Copyright 2002-2023 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -18,11 +18,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #if __MPFR_STDC (199901L)
@@ -62,7 +60,7 @@ special (void)
   /* coverage test */
   mpfr_set_prec (x, 2);
   mpfr_set_ui (x, 1, MPFR_RNDN);
-  mpfr_mul_2exp (x, x, mp_bits_per_limb, MPFR_RNDN);
+  mpfr_mul_2ui (x, x, mp_bits_per_limb, MPFR_RNDN);
   mpfr_rint (y, x, MPFR_RNDN);
   MPFR_ASSERTN(mpfr_cmp (y, x) == 0);
 
@@ -265,36 +263,42 @@ basic_tests (void)
   emax = mpfr_get_emax ();
 
   mpfr_init2 (x, 16);
-  for (prec = 2; prec <= 7; prec++)
+  for (prec = MPFR_PREC_MIN; prec <= 7; prec++)
     {
       mpfr_inits2 (prec, y, z, (mpfr_ptr) 0);
       for (s = 1; s >= -1; s -= 2)
         for (i = 1; i <= 72; i++)
           {
-            int k, t, u, v, f, e;
+            int k, t, u, v, f, e, b;
 
             for (t = i/4, k = 0; t >= 1 << prec; t >>= 1, k++)
               ;
+            b = !(t & 1);
             t <<= k;
             for (u = (i+3)/4, k = 0; u >= 1 << prec; u = (u+1)/2, k++)
               ;
             u <<= k;
             v = i < (t+u) << 1 ? t : u;
+            if (b)
+              b = i == (t+u) << 1;
             f = t == u ? 0 : i % 4 == 0 ? 1 : 2;
 
             mpfr_set_si_2exp (x, s * i, -2, MPFR_RNDN);
             e = mpfr_get_exp (x);
-            RND_LOOP(r)
+            RND_LOOP_NO_RNDF (r)
               {
                 BASIC_TEST (trunc, s * (i/4));
                 BASIC_TEST (floor, s > 0 ? i/4 : - ((i+3)/4));
                 BASIC_TEST (ceil, s > 0 ? (i+3)/4 : - (i/4));
                 BASIC_TEST (round, s * ((i+2)/4));
+                BASIC_TEST (roundeven, s * (i % 8 == 2 ? i/4 : (i+2)/4));
               }
             BASIC_TEST2 (trunc, s * t, - s * f);
             BASIC_TEST2 (floor, s > 0 ? t : - u, - f);
             BASIC_TEST2 (ceil, s > 0 ? u : - t, f);
             BASIC_TEST2 (round, s * v, v == t ? - s * f : s * f);
+            BASIC_TEST2 (roundeven, s * (b ? t : v),
+                         b || v == t ? - s * f : s * f);
           }
       mpfr_clears (y, z, (mpfr_ptr) 0);
     }
@@ -303,8 +307,23 @@ basic_tests (void)
 
 #if __MPFR_STDC (199901L)
 
+typedef int (*F2)(mpfr_ptr, mpfr_srcptr);
+
+/* The argument g below will be of type F2 with args (mpfr_ptr, mpfr_srcptr),
+   except for mpfr_rint, with args (mpfr_ptr, mpfr_srcptr, mpfr_rnd_t). So,
+   when passing mpfr_rint, we need a cast: "(F2) &mpfr_rint". The cast will
+   also be needed to compare the pointers: "g == (F2) &mpfr_rint", and that's
+   all. */
+
+#if defined(__GNUC__)
+#pragma GCC diagnostic push
+#if __GNUC__ >= 8 || __clang_major__ >= 13
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif
+#endif
+
 static void
-test_fct (double (*f)(double), int (*g)(), char *s, mpfr_rnd_t r)
+test_fct (double (*f)(double), F2 g, const char *s, mpfr_rnd_t r)
 {
   double d, y;
   mpfr_t dd, yy;
@@ -315,7 +334,7 @@ test_fct (double (*f)(double), int (*g)(), char *s, mpfr_rnd_t r)
     {
       mpfr_set_d (dd, d, r);
       y = (*f)(d);
-      if (g == &mpfr_rint)
+      if (g == (F2) &mpfr_rint)
         mpfr_rint (yy, dd, r);
       else
         (*g)(yy, dd);
@@ -333,12 +352,12 @@ test_fct (double (*f)(double), int (*g)(), char *s, mpfr_rnd_t r)
   mpfr_clear (yy);
 }
 
-#define TEST_FCT(F) test_fct (&F, &mpfr_##F, #F, r)
+#define TEST_FCT(F) test_fct (&F, &mpfr_##F, #F, (mpfr_rnd_t) r)
 
 static void
 test_against_libc (void)
 {
-  mpfr_rnd_t r = MPFR_RNDN;
+  int r = MPFR_RNDN;
 
   (void) r;  /* avoid a warning by using r */
 #if HAVE_ROUND
@@ -354,25 +373,28 @@ test_against_libc (void)
   TEST_FCT (ceil);
 #endif
 #if HAVE_NEARBYINT
-  for (r = 0; r < MPFR_RND_MAX ; r++)
-    if (mpfr_set_machine_rnd_mode (r) == 0)
-      test_fct (&nearbyint, &mpfr_rint, "rint", r);
+  RND_LOOP (r)
+    if (mpfr_set_machine_rnd_mode ((mpfr_rnd_t) r) == 0)
+      test_fct (&nearbyint, (F2) &mpfr_rint, "rint", (mpfr_rnd_t) r);
 #endif
 }
 
+#if defined(__GNUC__)
+#pragma GCC diagnostic pop
+#endif
+
 #endif
 
 static void
-err (const char *str, mp_size_t s, mpfr_t x, mpfr_t y, mpfr_prec_t p,
+err (const char *str, mp_size_t s, mpfr_ptr x, mpfr_ptr y, mpfr_prec_t p,
      mpfr_rnd_t r, int trint, int inexact)
 {
   printf ("Error: %s\ns = %u, p = %u, r = %s, trint = %d, inexact = %d\nx = ",
           str, (unsigned int) s, (unsigned int) p, mpfr_print_rnd_mode (r),
           trint, inexact);
-  mpfr_print_binary (x);
-  printf ("\ny = ");
-  mpfr_print_binary (y);
-  printf ("\n");
+  mpfr_dump (x);
+  printf ("y = ");
+  mpfr_dump (y);
   exit (1);
 }
 
@@ -465,6 +487,12 @@ coverage_03032011 (void)
 #define test_generic test_generic_round
 #include "tgeneric.c"
 
+#define TEST_FUNCTION mpfr_rint_roundeven
+#define TEST_RANDOM_EMIN -20
+#define TEST_RANDOM_ALWAYS_SCALE 1
+#define test_generic test_generic_roundeven
+#include "tgeneric.c"
+
 int
 main (int argc, char *argv[])
 {
@@ -484,86 +512,109 @@ main (int argc, char *argv[])
   mpfr_init (u);
   mpfr_init (v);
   mpz_set_ui (z, 1);
-  for (s = 2; s < 100; s++)
+  /* the code below works for 1 <= MPFR_PREC_MIN <= 2 */
+  MPFR_ASSERTN(1 <= MPFR_PREC_MIN && MPFR_PREC_MIN <= 2);
+  for (s = MPFR_PREC_MIN; s < 100; s++)
     {
-      /* z has exactly s bits */
-
-      mpz_mul_2exp (z, z, 1);
-      if (randlimb () % 2)
-        mpz_add_ui (z, z, 1);
+      if (s > 1)
+        {
+          mpz_mul_2exp (z, z, 1);
+          if (RAND_BOOL ())
+            mpz_add_ui (z, z, 1);
+        }
+      /* now 2^(s-1) <= z < 2^s */
       mpfr_set_prec (x, s);
       mpfr_set_prec (t, s);
       mpfr_set_prec (u, s);
       if (mpfr_set_z (x, z, MPFR_RNDN))
         {
-          printf ("Error: mpfr_set_z should be exact (s = %u)\n",
-                  (unsigned int) s);
+#ifndef MPFR_USE_MINI_GMP
+          gmp_printf ("Error: mpfr_set_z should be exact (z = %Zd, s = %u)\n",
+                      z, (unsigned int) s);
+#else /* mini-gmp has no gmp_printf (at least in gmp-6.1.2) */
+          printf ("mpfr_set_z should be exact\n");
+#endif
           exit (1);
         }
-      if (randlimb () % 2)
+      if (RAND_BOOL ())
         mpfr_neg (x, x, MPFR_RNDN);
-      if (randlimb () % 2)
+      if (RAND_BOOL ())
         mpfr_div_2ui (x, x, randlimb () % s, MPFR_RNDN);
-      for (p = 2; p < 100; p++)
+      for (p = MPFR_PREC_MIN; p < 100; p++)
         {
           int trint;
           mpfr_set_prec (y, p);
           mpfr_set_prec (v, p);
-          for (r = 0; r < MPFR_RND_MAX ; r++)
-            for (trint = 0; trint < 3; trint++)
+          RND_LOOP (r)
+            for (trint = 0; trint < 4; trint++)
               {
                 if (trint == 2)
                   inexact = mpfr_rint (y, x, (mpfr_rnd_t) r);
+                else if (trint == 3)
+                  {
+                    if (r != MPFR_RNDN)
+                      continue;
+                    inexact = mpfr_round (y, x);
+                  }
                 else if (r == MPFR_RNDN)
-                  inexact = mpfr_round (y, x);
+                  inexact = (trint ? mpfr_roundeven (y, x) :
+                             mpfr_rint_roundeven (y, x, MPFR_RNDZ));
                 else if (r == MPFR_RNDZ)
                   inexact = (trint ? mpfr_trunc (y, x) :
                              mpfr_rint_trunc (y, x, MPFR_RNDZ));
                 else if (r == MPFR_RNDU)
                   inexact = (trint ? mpfr_ceil (y, x) :
                              mpfr_rint_ceil (y, x, MPFR_RNDU));
-                else /* r = MPFR_RNDD */
+                else if (r == MPFR_RNDD)
                   inexact = (trint ? mpfr_floor (y, x) :
                              mpfr_rint_floor (y, x, MPFR_RNDD));
+                else
+                  {
+                    MPFR_ASSERTN (r == MPFR_RNDA || r == MPFR_RNDF);
+                    continue;
+                  }
                 if (mpfr_sub (t, y, x, MPFR_RNDN))
-                  err ("subtraction 1 should be exact",
-                       s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                  err ("subtraction 1 should be exact", s, x, y, p,
+                       (mpfr_rnd_t) r, trint, inexact);
                 sign_t = mpfr_cmp_ui (t, 0);
                 if (trint != 0 &&
                     (((inexact == 0) && (sign_t != 0)) ||
                      ((inexact < 0) && (sign_t >= 0)) ||
                      ((inexact > 0) && (sign_t <= 0))))
-                  err ("wrong inexact flag", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                  err ("wrong inexact flag", s, x, y, p,
+                       (mpfr_rnd_t) r, trint, inexact);
                 if (inexact == 0)
                   continue; /* end of the test for exact results */
 
-                if (((r == MPFR_RNDD || (r == MPFR_RNDZ && MPFR_SIGN (x) > 0))
+                if (((r == MPFR_RNDD || (r == MPFR_RNDZ && MPFR_IS_POS (x)))
                      && inexact > 0) ||
-                    ((r == MPFR_RNDU || (r == MPFR_RNDZ && MPFR_SIGN (x) < 0))
+                    ((r == MPFR_RNDU || (r == MPFR_RNDZ && MPFR_IS_NEG (x)))
                      && inexact < 0))
-                  err ("wrong rounding direction",
-                       s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                  err ("wrong rounding direction", s, x, y, p,
+                       (mpfr_rnd_t) r, trint, inexact);
                 if (inexact < 0)
                   {
                     mpfr_add_ui (v, y, 1, MPFR_RNDU);
                     if (mpfr_cmp (v, x) <= 0)
-                      err ("representable integer between x and its "
-                           "rounded value", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                      err ("representable integer between x and "
+                           "its rounded value", s, x, y, p,
+                           (mpfr_rnd_t) r, trint, inexact);
                   }
                 else
                   {
                     mpfr_sub_ui (v, y, 1, MPFR_RNDD);
                     if (mpfr_cmp (v, x) >= 0)
-                      err ("representable integer between x and its "
-                           "rounded value", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                      err ("representable integer between x and "
+                           "its rounded value", s, x, y, p,
+                           (mpfr_rnd_t) r, trint, inexact);
                   }
-                if (r == MPFR_RNDN)
+                if (r == MPFR_RNDN && trint != 0)
                   {
                     int cmp;
                     if (mpfr_sub (u, v, x, MPFR_RNDN))
-                      err ("subtraction 2 should be exact",
-                           s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
-                    cmp = mpfr_cmp_abs (t, u);
+                      err ("subtraction 2 should be exact", s, x, y, p,
+                           (mpfr_rnd_t) r, trint, inexact);
+                    cmp = mpfr_cmpabs (t, u);
                     if (cmp > 0)
                       err ("faithful rounding, but not the nearest integer",
                            s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
@@ -577,28 +628,35 @@ main (int argc, char *argv[])
                            mode: round to an even integer or significand. */
                         mpfr_div_2ui (y, y, 1, MPFR_RNDZ);
                         if (!mpfr_integer_p (y))
-                          err ("halfway case for mpfr_rint, result isn't an"
-                               " even integer", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
-                        /* If floor(x) and ceil(x) aren't both representable
-                           integers, the significand must be even. */
-                        mpfr_sub (v, v, y, MPFR_RNDN);
-                        mpfr_abs (v, v, MPFR_RNDN);
-                        if (mpfr_cmp_ui (v, 1) != 0)
+                          err ("halfway case for mpfr_rint, result isn't "
+                               "an even integer", s, x, y, p,
+                               (mpfr_rnd_t) r, trint, inexact);
+                        if (p > 1)
                           {
-                            mpfr_div_2si (y, y, MPFR_EXP (y) - MPFR_PREC (y)
-                                          + 1, MPFR_RNDN);
-                            if (!mpfr_integer_p (y))
-                              err ("halfway case for mpfr_rint, significand isn't"
-                                   " even", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                            /* For p > 1, if floor(x) and ceil(x) aren't
+                               both representable integers, the significand
+                               must be even. */
+                            mpfr_sub (v, v, y, MPFR_RNDN);
+                            mpfr_abs (v, v, MPFR_RNDN);
+                            if (mpfr_cmp_ui (v, 1) != 0)
+                              {
+                                mpfr_div_2si (y, y, MPFR_EXP (y) -
+                                              MPFR_PREC (y) + 1, MPFR_RNDN);
+                                if (!mpfr_integer_p (y))
+                                  err ("halfway case for mpfr_rint, "
+                                       "significand isn't even", s, x, y, p,
+                                       (mpfr_rnd_t) r, trint, inexact);
+                              }
                           }
                       }
-                    else
+                    else if (trint == 3)
                       { /* halfway case for mpfr_round: x must have been
                            rounded away from zero. */
-                        if ((MPFR_SIGN (x) > 0 && inexact < 0) ||
-                            (MPFR_SIGN (x) < 0 && inexact > 0))
-                          err ("halfway case for mpfr_round, bad rounding"
-                               " direction", s, x, y, p, (mpfr_rnd_t) r, trint, inexact);
+                        if ((MPFR_IS_POS (x) && inexact < 0) ||
+                            (MPFR_IS_NEG (x) && inexact > 0))
+                          err ("halfway case for mpfr_round, "
+                               "bad rounding direction", s, x, y, p,
+                               (mpfr_rnd_t) r, trint, inexact);
                       }
                   }
               }
@@ -615,10 +673,11 @@ main (int argc, char *argv[])
   basic_tests ();
   coverage_03032011 ();
 
-  test_generic_trunc (2, 300, 20);
-  test_generic_floor (2, 300, 20);
-  test_generic_ceil (2, 300, 20);
-  test_generic_round (2, 300, 20);
+  test_generic_trunc (MPFR_PREC_MIN, 300, 20);
+  test_generic_floor (MPFR_PREC_MIN, 300, 20);
+  test_generic_ceil (MPFR_PREC_MIN, 300, 20);
+  test_generic_round (MPFR_PREC_MIN, 300, 20);
+  test_generic_roundeven (MPFR_PREC_MIN, 300, 20);
 
 #if __MPFR_STDC (199901L)
   if (argc > 1 && strcmp (argv[1], "-s") == 0)
